<?php defined("SYSPATH") or die("No direct script access.");

class approvalqueue_Controller extends Controller {

  function dialog($item_id) {
    $item = ORM::factory("item",$item_id);
    $v = new View("approval_dialog.html");
    $v->item = $item;
    $v->root = item::root();
    access::required("view", $item);
    access::required("edit", $item);
    print $v;
  }

  function move_it($item_id) {
    $move = ORM::factory("appq")
      ->where("appq_item", "=", $item_id)
      ->find();
    
    if(!$move->loaded()){
      message::warning(t("ERR: Can't see destination album - perhaps this wasn't uploaded for approval!"));
      json::reply(array("result" => "success", "reload" => 1));
      return;
    }
 

   $dst_album = $move->appq_album;

    $item = ORM::factory("item",$item_id);
    $dest = ORM::factory("item",$dst_album);
    ($item->is_photo() ? $what = "photo" : $what = "movie");

    item::move($item, $dest);

    db::build()->delete("appqs")
      ->where("appq_item", "=", $item_id)
      ->execute();

    $bc = approvalqueue::breadcrumb($item_id);

    message::success(t("The $what was approved and posted to: ").$bc);
    json::reply(array("result" => "success", "reload" => 1));
    return;
  }

  function add_it($appq_dest) {
    $appq_album = module::get_var("approvalqueue","appq_album");
    $album = ORM::factory("item", $appq_album);
    $view = new View("approvalqueue_additem.html");
    $view->album = $album;
    $view->appq_album = $appq_album;
    $view->appq_dest = $appq_dest;
    print $view;
  }

  function add_item($appq_dest) {
    $title = Input::instance()->post("title");
    $desc = Input::instance()->post("desc");
    $comment = Input::instance()->post("comment");
    $appq_album = module::get_var("approvalqueue","appq_album");
    if($_FILES["file"]["error"] > 0) {
      message::warning(t("Error, nothing uploaded!"));
      json::reply(array("result" => "success"));
      return;
    }
    $tfname = $_FILES["file"]["name"];
    $dbfile = $_FILES["file"]["tmp_name"];
    $album = ORM::factory("item", $appq_album);
    access::required("add", $album);

    $parts = preg_split("/\./",$tfname);
    $ext = array_pop($parts);
    $fileerr = 0;
    if(!legal_file::get_extensions($ext)) {
      message::warning(t("$tfname was not added - invalid file type"));
      $fileerr = 1;
    } else {
      $fpath = VARPATH."uploads/";
      $fname = $fpath.$tfname;
      move_uploaded_file($dbfile, $fname);
    }
    if($fileerr == 1){
      message::error(t("No valid items uploaded"));
      json::reply(array("result" => "success"));
      return;
    }
    $added_type = "- ERROR -";
    try {
        $item = ORM::factory("item");
        $item->name = basename($fname);
	if($title){
	  $item->title = $title;
	} else {
	  $item->title = item::convert_filename_to_title($item->name);
	}
	$item->description = $desc;
        $item->parent_id = $album->id;
        $item->set_data_file($fname);
        $path_info = @pathinfo($fname);
        if(array_key_exists("extension", $path_info) &&
           in_array(strtolower($path_info["extension"]), array("flv", "mp4", "m4v"))) {
          $item->type = "movie";
          $item->save();
          $added_type = "movie";
	} else {
          $item->type = "photo";
          $item->save();
          $added_type = "photo";
	}
    } catch (Exception $e) {
        Kohana_Log::add("error", $e->getMessage(), "\n" . $e->getTraceAsString());
        if ($e instanceof ORM_Validation_exception) {
            Kohana_Log::add("error", "Validation errors ($fname): " . print_r($e->validation->errors(), 1));
	}
    }
    
    if (file_exists($fname)) {
      unlink($fname);
    }
    message::success(t("Added a ".$added_type." for approval."));

    db::build()->insert(
	"appqs",
	array("appq_item" => $item->id,
	      "appq_album" => $appq_dest))
    ->execute();
    
    db::build()->update("appqs")
      ->set(array("comment" => $comment))
      ->where("appq_item", "=", $item->id)
      ->execute();

    json::reply(array("result" => "success"));
    return;
  }

  function fix_appq_perms($what){
    $test = ORM::factory("item",module::get_var("approvalqueue","appq_album"));
    $reg = identity::lookup_group("2");
    $eb = identity::lookup_group("1");
    access::allow($reg, "add", $test);
    access::allow($reg, "edit", $test);
    access::deny($reg, "view", $test);
    access::deny($reg, "view_full", $test);
    access::deny("$eb", "add", $test);
    access::deny("$eb", "edit", $test);
    access::deny("$eb", "view", $test);
    access::deny("$eb", "view_full", $test);

    if($what == "fix" || $what == "add") {
      message::success(t("Approval Queue album permissions fixed"));
      json::reply(array("result" => "success", "reload" => 1));
    }
    return;
  }
  
  function create_appq_album() {
    $album = ORM::factory("item");
    $album->type = "album";
    $album->parent_id = 1;
    $album->name = "Approval-Queue";
    $album->title = "Approval-Queue";
    $album->slug = "Approval-Queue";
    $album->description = "Approval-Queue Album";
    $album->validate();
    $album->save();
    
    module::set_var("approvalqueue","appq_album",$album->id);

    fix_appq_perms("partofalbumfix");

    message::success(t("Approval Queue album created"));
    json::reply(array("result" => "success", "reload" => 1));
    return;
  }

  function create_appqs_db() {
    $db = Database::instance();
    $db->query("CREATE TABLE IF NOT EXISTS {appqs} (
	`id` int(9) NOT NULL auto_increment,
	`appq_item` int(9) NOT NULL,
	`appq_album` int(9) NOT NULL,
	PRIMARY KEY (`id`),
	KEY(`appq_item`, `id`))
	DEFAULT CHARSET=utf8;");
  }

}
